## The breast cancer data pre-processing.
##This code generates the files dt_cancer.csv and dt_no_cancer.csv.
##These two data files can also be found in the same folder as this file.
## They are also available in matlab format as dtcancer.mat and dtnocancer.mat

##The original data Table_ClinicalData_short.csv, Table_NKI_295_1.txt, Table_NKI_295_2.txt, 
## Table_NKI_295_3.txt, Table_NKI_295_4.txt, Table_NKI_295_5.txt, Table_NKI_295_6.txt
## that are processed here can be download from http://ccb.nki.nl/data/

datapath = './breast_cancer_data/';
dt = read.csv(paste(datapath,'Table_ClinicalData_short.csv',sep=''), header = T);
Y = as.integer(dt[,2]); #whether survive 2 years without relaps

## INTENSITY BASED GENE-WISE SELECTION ##
## get all() the genes' intensities ##
Table_txt1 <- read.table(file = paste(datapath,"Table_NKI_295_1.txt", sep=''), header = FALSE, sep = "\t")
  i=6
  tgn1 = cbind(Table_txt1$V1, Table_txt1$V2)
  while(i <= dim(Table_txt1)[2]){
    mycol = paste("V", i, sep="")
    tgn1 = cbind(tgn1,Table_txt1[mycol])
    i = i+5
  }

Table_txt2 <- read.table(file = paste(datapath,"Table_NKI_295_2.txt", sep=''), header = FALSE, sep = "\t")
i=6
while(i <= dim(Table_txt2)[2]){
  mycol = paste("V", i, sep="")
  tgn1 = cbind(tgn1,Table_txt2[mycol])
  i = i+5
}

Table_txt3 <- read.table(file = paste(datapath,"Table_NKI_295_3.txt", sep=''), header = FALSE, sep = "\t")
i=6
while(i <= dim(Table_txt3)[2]){
  mycol = paste("V", i, sep="")
  tgn1 = cbind(tgn1,Table_txt3[mycol])
  i = i+5
}

Table_txt4 <- read.table(file = paste(datapath,"Table_NKI_295_4.txt", sep=''), header = FALSE, sep = "\t")
i=6
while(i <= dim(Table_txt4)[2]){
  mycol = paste("V", i, sep="")
  tgn1 = cbind(tgn1,Table_txt4[mycol])
  i = i+5
}

Table_txt5 <- read.table(file = paste(datapath,"Table_NKI_295_5.txt", sep=''), header = FALSE, sep = "\t")
i=6
while(i <= dim(Table_txt5)[2]){
  mycol = paste("V", i, sep="")
  tgn1 = cbind(tgn1,Table_txt5[mycol])
  i = i+5
}

Table_txt6 <- read.table(file = paste(datapath,"Table_NKI_295_6.txt", sep=''), header = FALSE, sep = "\t")
i=6
while(i <= dim(Table_txt6)[2]){
  mycol = paste("V", i, sep="")
  tgn1 = cbind(tgn1,Table_txt6[mycol])
  i = i+5
}

Table.gene_intensity = as.matrix(tgn1[3:dim(tgn1)[1], 3:dim(tgn1)[2]])
colnames(Table.gene_intensity) = NULL
Table.gene_intensity = t(Table.gene_intensity)
class(Table.gene_intensity) = "numeric";
colnames(Table.gene_intensity) = as.character(Table_txt1[3:dim(Table_txt1)[1],1])


## node(gene)-wise regression to EVENTmeta in the Clinical dataset ##
Genewise.pval = function(j){
  glm.onegene = glm(Y ~ as.numeric(Table.gene_intensity[,j]),  family = binomial())
  pval.onegene = summary(glm.onegene)$coefficients[2,4]
  return(pval.onegene)
}
pval_of_genes = sapply(1:dim(Table.gene_intensity)[2], function(j) Genewise.pval(j), simplify = T)

## pick genes by p-value greater than 0.005
thresh = 0.005;
picked.genes.id = which(pval_of_genes<=thresh)
picked.genes.name = colnames(Table.gene_intensity)[picked.genes.id]
picked.gene_intensity = Table.gene_intensity[,picked.genes.name]

## screening for Outliers following Song et al 2014. Remove all genes w. more than 2 outliers
##sample_size = length(picked.gene_intensity[,1])
##med_value = apply(picked.gene_intensity,2,median)
##med_deviation = apply(picked.gene_intensity,2,function(x) median(abs(x-median(x))))
##med_value_mat = matrix(rep(med_value,sample_size),nrow = sample_size,byrow=T)
##med_deviation_mat = matrix(rep(med_deviation,sample_size),nrow = sample_size,byrow=T)
##Z_value = 0.6745*(abs(picked.gene_intensity-med_value_mat)/med_deviation_mat);
##selected_genes = apply(Z_value,2, function(x) sum(x>3.5) <= 2)

selected.genes.name = picked.genes.name
selected.gene.intensity = picked.gene_intensity

main_dt = selected.gene.intensity;
main_dt_cancer = main_dt[Y==1,];
main_dt_no_cancer = main_dt[Y==0,];

#Center and rescale each variable
for (j in 1:length(main_dt_cancer[1,])){
  m = mean(main_dt_cancer[,j]);
  s = sqrt(var(main_dt_cancer[,j]));
  main_dt_cancer[,j] = (main_dt_cancer[,j] - m)/s;
}

for (j in 1:length(main_dt_no_cancer[1,])){
  m = mean(main_dt_no_cancer[,j]);
  s = sqrt(var(main_dt_no_cancer[,j]));
  main_dt_no_cancer[,j] = (main_dt_no_cancer[,j] - m)/s;
}

#save to file
gene_names = selected.genes.name;
write.csv(main_dt_cancer,'dt_cancer.csv',row.names=F,col.names=F);
write.csv(main_dt_no_cancer,'dt_no_cancer.csv',row.names=F,col.names=F);
write.csv(gene_names,'gene_names.csv',row.names=F,col.names=F);



########### THE END ######################################

